@@ -22,6 +22,8 @@ install("nltk")
 install("textblob")
 install("fastdtw")
 install ("spicy")
+install("cdlib")
+install("community")
 ##----------------------------------------------IMPORTS---------------------------------------------------------------------------#	
 import json
 import itertools
@@ -42,6 +44,9 @@ from os import path
 import os
 from datetime import timedelta
 from datetime import datetime
+import community as community_louvain
+#from cdlib[C] import algorithms
+
 
 #---------------------------------------------NLP-Part-------------------------------------------------------------------------------#
 from textblob import TextBlob
@@ -114,7 +119,28 @@ def building_node_list(df):
     node_list = node_list.drop_duplicates()
     node_list = node_list[node_list["screen_name"] !="Empty"].reset_index()
     return list(node_list["screen_name"])
+    
+    
+#---------------------------------Undirect Graph ------------------------------------------------
+def buildingGraph(df_graph):
+    G =nx.Graph()
+    for i in range(len(df_graph)):
+        G.add_edges_from([(df_graph.screen_name.iloc[i], df_graph.screen_name_mention.iloc[i])])
+    return G
 
+def community_algortihm(df_graph):
+
+    G = buildingGraph(df_graph)
+    # compute the best partition
+    partition = community_louvain.best_partition(G)
+    l=[]
+    for (key,value) in partition.items():
+        l.append([key, value])
+
+    df_algComu=pd.DataFrame(data=l,columns= ['screen_name','community'])
+    return df_algComu
+
+#------------------------------Directed Graph ---------------------------------------------------
 def building_DiGraph(node_list, df):
     DiG = nx.DiGraph()
     DiG.add_nodes_from(node_list)
@@ -226,7 +252,29 @@ def graph_features(DiG,df):
     df_nodes_self_loop_iteration = self_loop_iteration_nodes(df)
     output = merge_dataframes(output, df_nodes_self_loop_iteration)
     return output
-
+    
+def graph_json_values(df_graph, df_algoComu):
+    
+    data = {}
+    data['nodes'] = []
+    for i in range(len(df_algComu)):
+      
+      data['nodes'].append({
+          'screen_name': df_algComu["screen_name"].iloc[i],
+          'community': str(df_algComu["community"].iloc[i])
+      })
+      
+    data['edges'] = []
+    for i in range(len(df_graph)):
+      
+      data['edges'].append({
+          'screen_name': df_graph["screen_name"].iloc[i],
+          'screen_name_mention': df_graph["screen_name_mention"].iloc[i],
+          'iteration': str(df_graph["iteration"].iloc[i])
+      })
+      
+    return json.dumps(data)
+    
 def user_timeline_extraction(node_list, consumerKey, consumerSecret,accessToken,accessTokenSecret,filepath, filename,n_groups=1,save_by=300):
     # get_user Returns the 20 most recent statuses posted from the authenticating user or the user specified. 
     profile_iter = save_by
@@ -300,14 +348,14 @@ def user_timeline_extraction(node_list, consumerKey, consumerSecret,accessToken,
         if path.exists(os.path.join(filepath,filename)):
           df2.to_csv(path_or_buf =os.path.join(filepath,filename),mode="a", index=False, header=False)
         else:
-          df2.to_csv(path_of_buf = os.path.join(filepath,filename),index=False)         
+          df2.to_csv(path_or_buf = os.path.join(filepath,filename),index=False)         
         if len(list_users_noInfo) !=0:
           nodes = pd.DataFrame()
           nodes["nodos"] = list_users_noInfo
           if path.exists(os.path.join(filepath,"node_without_",filename)):
-            nodes.to_csv(os.path.join(filepath,"node_without_",filename), mode ="a",index=False, header=False)          
+            nodes.to_csv(path_or_buf = os.path.join(filepath,"node_without_"+filename), mode ="a",index=False, header=False)          
           else:
-             nodes.to_csv(os.path.join(filepath,"node_without_",filename), index=False)
+             nodes.to_csv(path_or_buf = os.path.join(filepath,"node_without_"+filename), index=False)
         k = k+profile_iter        
         if ultima ==1:          
           ultima =2
@@ -630,45 +678,47 @@ def time_series_tw (df):
     return df 
 
 #-------------------------------------------Ratios and other advanced features----------------------------------#
-def duplicates_detection(data, i):
-    if i < len(data):
-        possible_duplicated = []
-        for j in range(len(data)):
-            if j !=i:
-                list_dtw_1 = np.array(data["tw_per_day_list"].iloc[i])              
-                list_dtw_2 = np.array(data["tw_per_day_list"].iloc[j])
-                distance, path = fastdtw(list_dtw_1,list_dtw_2)
-                if (distance < 600):
-                    possible_duplicated.append({"screen_name":data["screen_name"].iloc[j], "distance": distance})
-        if (len(possible_duplicated)==0):
-            possible_duplicated.append({"screen_name": "Empty", "distance": "Empty", "tw_similarity": 0,"tw_match_tw": "Empty"})   
-            data["possible_duplicates"].iloc[i] = possible_duplicated        
-        else: 
-            data["possible_duplicates"].iloc[i] = possible_duplicated          
-            for t in range(len(data["possible_duplicates"].iloc[i])):
-                #text_1 is the differences possible duplicates with the profile evaluate
-                text_1_aux = pd.DataFrame({"text":data[data["screen_name"] == data["possible_duplicates"].iloc[i][t]["screen_name"]]["text_tweets_days_list"]})
-                text_1 = pd.DataFrame()
-                for a in range(len(text_1_aux)):
-                    aux2 = pd.DataFrame({'text':text_1_aux["text"].iloc[a]})
-                    text_1 = text_1.append(aux2, ignore_index=False).reset_index(drop=True)
-                text_1 = list(itertools.chain(*text_1["text"]))   
-                text_2 = list(itertools.chain(*data.iloc[i]["text_tweets_days_list"]))               
-                if "Empty" in text_1:
-                  text_1 = list(filter(("Empty").__ne__, text_1))
-                if "Empty" in text_2:  
-                  text_2 = list(filter(("Empty").__ne__, text_2))
-                if "" in text_1:
-                  text_1 = list(filter(("").__ne__, text_1))
-                if "" in text_2:  
-                  text_2 = list(filter(("").__ne__, text_2))  
-                comparacion = [item for item in text_2 if item in text_1]
-                if len(comparacion) > 0:
-                    data["possible_duplicates"].iloc[i][t]["tw_similarity"] = len(comparacion)/(data["num_userTw"].iloc[i])
-                    data["possible_duplicates"].iloc[i][t]["text_match_tw"] = comparacion
-                else:
-                    data["possible_duplicates"].iloc[i][t]["tw_similarity"] = 0
-                    data["possible_duplicates"].iloc[i][t]["text_match_tw"] = "Empty"
+def duplicates_detection(data):
+    data["possible_duplicates"] = 0
+    for i in range(len(data)):
+        if i < len(data):
+            possible_duplicated = []
+            for j in range(len(data)):
+                if j !=i:
+                    list_dtw_1 = np.array(data["tw_per_day_list"].iloc[i])              
+                    list_dtw_2 = np.array(data["tw_per_day_list"].iloc[j])
+                    distance, path = fastdtw(list_dtw_1,list_dtw_2)
+                    if (distance < 600):
+                        possible_duplicated.append({"screen_name":data["screen_name"].iloc[j], "distance": distance})
+            if (len(possible_duplicated)==0):
+                possible_duplicated.append({"screen_name": "Empty", "distance": "Empty", "tw_similarity": 0,"tw_match_tw": "Empty"})   
+                data["possible_duplicates"].iloc[i] = possible_duplicated        
+            else: 
+                data["possible_duplicates"].iloc[i] = possible_duplicated          
+                for t in range(len(data["possible_duplicates"].iloc[i])):
+                    #text_1 is the differences possible duplicates with the profile evaluate
+                    text_1_aux = pd.DataFrame({"text":data[data["screen_name"] == data["possible_duplicates"].iloc[i][t]["screen_name"]]["text_tweets_days_list"]})
+                    text_1 = pd.DataFrame()
+                    for a in range(len(text_1_aux)):
+                        aux2 = pd.DataFrame({'text':text_1_aux["text"].iloc[a]})
+                        text_1 = text_1.append(aux2, ignore_index=False).reset_index(drop=True)
+                    text_1 = list(itertools.chain(*text_1["text"]))   
+                    text_2 = list(itertools.chain(*data.iloc[i]["text_tweets_days_list"]))               
+                    if "Empty" in text_1:
+                      text_1 = list(filter(("Empty").__ne__, text_1))
+                    if "Empty" in text_2:  
+                      text_2 = list(filter(("Empty").__ne__, text_2))
+                    if "" in text_1:
+                      text_1 = list(filter(("").__ne__, text_1))
+                    if "" in text_2:  
+                      text_2 = list(filter(("").__ne__, text_2))  
+                    comparacion = [item for item in text_2 if item in text_1]
+                    if len(comparacion) > 0:
+                        data["possible_duplicates"].iloc[i][t]["tw_similarity"] = len(comparacion)/(data["num_userTw"].iloc[i])
+                        data["possible_duplicates"].iloc[i][t]["text_match_tw"] = comparacion
+                    else:
+                        data["possible_duplicates"].iloc[i][t]["tw_similarity"] = 0
+                        data["possible_duplicates"].iloc[i][t]["text_match_tw"] = "Empty"
     return data
 	
 	
@@ -696,6 +746,9 @@ def advanced_features(data):
     data["follow_rate"][data["followers"]== 0] = np.median(data["follow_rate"])
     
     for i in range(len(data)):
+        data["favorite_tweets_list"].iloc[i] = ast.literal_eval(data["favorite_tweets_list"].iloc[i])
+        data["retweet_tweets_list"].iloc[i] = ast.literal_eval(data["retweet_tweets_list"].iloc[i])
+        data["tw_per_day_list"].iloc[i] = ast.literal_eval(data["tw_per_day_list"].iloc[i])
         if (len(str(data.iloc[i]["screen_name"])) > 4):
             if((data.iloc[i]["screen_name"][-1].isdigit()) & (data.iloc[i]["screen_name"][-2].isdigit()) & 
                 (data.iloc[i]["screen_name"][-3].isdigit()) &(data.iloc[i]["screen_name"][-4].isdigit())):
@@ -720,12 +773,10 @@ def advanced_features(data):
         else:
             data["max_ret_tw"].iloc[i] = max((data["retweet_tweets_list"].iloc[i]))
           
-        data = duplicates_detection(data, i)
         data["max_tw_day"].iloc[i] =max((data["tw_per_day_list"].iloc[i]))
         data["days_with_tw"].iloc[i] = len(data["tw_per_day_list"].iloc[i])
         data["index_max_day_tw"].iloc[i] = data["tw_per_day_list"].iloc[i].index(data["max_tw_day"].iloc[i])        
     data = trend_tweets_features(data) 
-    data = close_duplicates(data)
         
     return data
 	
@@ -777,9 +828,6 @@ def close_duplicates(data):
 	
 def text_to_list_features_conversor (data):
 	for index, row in data.iterrows():
-		data["favorite_tweets_list"].iloc[index] = ast.literal_eval(data.loc[index, "favorite_tweets_list"])
-		data["retweet_tweets_list"].iloc[index] = ast.literal_eval(data.loc[index,"retweet_tweets_list"])
-		data["tw_per_day_list"].iloc[index] = ast.literal_eval(data.loc[index,"tw_per_day_list"])
 		data["text_tweets_days_list"].iloc[index] = data.loc[index,"text_tweets_days_list"].replace('"', "'").replace("], [","?#??").strip("[]'").split("?#??")
 		list_day = []                                                                                
 		list_text_tweets_day = []
@@ -885,70 +933,6 @@ def abnormal_categories_profiles(data_join):
         
     return data_join
 
-
-def abnormal_categories_profiles2(data_join):
-
-    
-    data_join["evaluate_NLP"] = "No"
-    for i in range(len(data_join)):
-        abnormal_category_belonging = []
-        #1.Profiles old spreader
-        if ((data_join["index_max_day_tw"].iloc[i] >=30) & (data_join["profile_type"].iloc[i]==False) &
-                      (data_join["max_tw_day"].iloc[i]>60)):
-            abnormal_category_belonging.append("Old Spreader")  
-        
-        #2.Profiles with low number of tweets but with a high iteration
-        if((data_join["trend"].iloc[i] == 0) & (data_join["profile_type"].iloc[i]==False) &
-                      (data_join["max_tw_day"].iloc[i]<40) & (data_join["tw_RT_rate"].iloc[i]<0.2) &
-                     ((data_join["max_ret_tw"].iloc[i]>500) | (data_join["max_fav_tw"].iloc[i]>500))):
-            abnormal_category_belonging.append("LRT_HI")
-            
-        #3.Profiles with a high number of RT in a specific day (try to spread information)
-        if((data_join["trend"].iloc[i] == 0) & (data_join["profile_type"].iloc[i]==False) &
-                      (data_join["max_tw_day"].iloc[i]>150) & (data_join["tw_RT_rate"].iloc[i]<0.1)):
-            abnormal_category_belonging.append("HRT_SD")
-            
-        #4.Possible influencer profile
-        if((data_join["trend"].iloc[i]==1) & (data_join["profile_type"].iloc[i]==False) &
-                 (data_join["in_degree"].iloc[i]>=5) & (data_join["followers"].iloc[i]>=3000)):
-            abnormal_category_belonging.append("Influencer")
-        
-        #5.Possible constant spreader
-        if((data_join["trend"].iloc[i]==1) & (data_join["profile_type"].iloc[i]==False) &
-                 (data_join["out_degree"].iloc[i]>=2) & (data_join["following"].iloc[i]>=3000)):
-            abnormal_category_belonging.append("Constant Spreader")                
-
-        #6.Possible new account with a high activity last days
-        if((data_join["trend"].iloc[i]==2) & (data_join["profile_type"].iloc[i]==False) &
-                 (data_join["twitter_years"].iloc[i]==1) & (data_join["max_tw_day"].iloc[i]>210)):
-            abnormal_category_belonging.append("NP_HA")                     
-
-        #7.Possible fake behaviour profile in the last days
-        if((data_join["trend"].iloc[i]==2) & (data_join["profile_type"].iloc[i]==False) &
-                 (data_join["fake_sum"].iloc[i]>=2)):
-            abnormal_category_belonging.append("Fake Behaviour") 
-        
-        #8.Possible influencers profile in the last days with a high iteration
-        if((data_join["trend"].iloc[i]==2) & (data_join["profile_type"].iloc[i]==False) &
-                 ((data_join["max_ret_tw"].iloc[i]>500) | (data_join["max_fav_tw"].iloc[i]>500)) &
-                      (data_join["max_tw_day"].iloc[i]>210)):
-            abnormal_category_belonging.append("Influ_LD_HI")
-        
-        #9.Profiles BOT (more than tweet or RT each 5 minutes the 24 hours)
-        if((data_join["max_tw_day"].iloc[i]>288) &(data_join["profile_type"].iloc[i]==False)):
-            abnormal_category_belonging.append("BOT")
-                                 
-        #10. More data required
-        if((data_join["trend"].iloc[i]==2) & (data_join["profile_type"].iloc[i]==False) & 
-                                  (data_join["max_tw_day"].iloc[i]>180)):
-            abnormal_category_belonging.append("More Data")
-        
-        if len(abnormal_category_belonging)!=0:
-            data_join["evaluate_NLP"].iloc[i] = abnormal_category_belonging          
-        
-    return data_join
-
-
 	
 def more_tweets_required2(df,df_graph_features,consumerKey,consumerSecret,accessToken,accessTokenSecret,filepath,filename,n_groups,save_by):
 	df_evaluate = df[(df["evaluate_NLP"] !="No") & (df["evaluate_NLP"] !="More Data")]
@@ -965,7 +949,6 @@ def more_tweets_required2(df,df_graph_features,consumerKey,consumerSecret,access
 	return df_evaluate
 	
 def more_tweets_required(df,df_graph_features,consumerKey,consumerSecret,accessToken,accessTokenSecret,filepath,filename,n_groups,save_by):
-    print(len(df))
     df_evaluate = df.copy()
     node_list_moreData = []
     delete_rows = []
@@ -976,11 +959,10 @@ def more_tweets_required(df,df_graph_features,consumerKey,consumerSecret,accessT
         if "No" in df["evaluate_NLP"].iloc[i]:
             delete_rows.append(i)
     df_evaluate= df_evaluate.drop(df_evaluate.index[delete_rows]).reset_index(drop=True)
-    print(len(df_evaluate))
     if len(node_list_moreData) >0:
         df_metadata_moreData, list_more_Data_noData = user_timeline_extraction(node_list_moreData,consumerKey,consumerSecret,accessToken,accessTokenSecret,filepath, filename,n_groups,save_by)
-        df_metadata_moreData = pd.read_csv(filepath + filename)
-        df_metadata_moreData = text_to_list_features_conversor(df_metadata_moreData)
+        df_metadata_moreData = pd.read_csv(os.path.join(filepath,filename))
+        #df_metadata_moreData = text_to_list_features_conversor(df_metadata_moreData)
         df_metadata_moreData = advanced_features(df_metadata_moreData)
         df_metadata_moreData = merge_dataframes(df_metadata_moreData,df_graph_features)
         df_metadata_moreData = abnormal_categories_profiles(df_metadata_moreData)
@@ -995,7 +977,7 @@ def more_tweets_required(df,df_graph_features,consumerKey,consumerSecret,accessT
     return df_evaluate
 
 def contextual_analysis_profile(data, keyword_corpus, hashtag_corpus):  
-    #To remember which name of t he tw_per_day_list belongs each tweets
+    #To remember which name of the tw_per_day_list belongs each tweets
     n_list_tw = range(0, len(data["tw_per_day_list"]))
     aux = pd.DataFrame({'text':data["text_tweets_days_list"],'RT':data["RT_days_list"],'lang': data["lang_days_list"], 'n_list_tw':n_list_tw})
     tweets_df = pd.DataFrame()
